/*  Web Designer
   Copyright (C) 1999  LADISLAS Willhem

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   Author's email : zillprog@freesurf.fr

   snailmail :      LADISLAS Willhem
   2, rue du Serpolet
   95490 VAUREAL
   FRANCE
 */
/* Functions in this file :
   add_afile_prj
 */

#include "webdesigner.h"


void
add_afile_prj (GtkWidget * w, GtkFileSelection * fs)
{

  gchar *path;

  path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

/* checks if it a directory path instead of a file path */
  if (path[strlen (path) - 1] == '/')
    {
      merror ("No file has been selected !");
      return;
    }

  numprjfiles = numprjfiles + 1;
  sprintf (projectfiles[numprjfiles].path, "%s", path);

  if (prjsaved == TRUE)
    prjsaved = FALSE;

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }

}				/* add_afile_prj() */


void
add_afile_prj_dialog (void)
{
  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened");
      return;
    }

  filew = gtk_file_selection_new ("Add a file...");

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", (GtkSignalFunc) add_afile_prj, filew);

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_signal_connect_object_after (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  /* Let's set the default filename */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "*.htm;*.html;*.shtml");

  gtk_widget_show (filew);

}				/* fin de add_afile_prj_dialog(void) */



void
new_project (void)
{
  char bufferp[255];

  if (prjsaved == FALSE)
    {
      merror("Current project has not been saved or closed");
      return;
    }

  sprintf (projectname, "%s", "new");
  numprjfiles = 0;
  prjsaved = FALSE;
  prjopnd = TRUE;
  prjnew = TRUE;

  sprintf (bufferp, "Project  : %s", projectname);
  gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, bufferp);

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
}				/* fin de new_project(void) */

void
save_project_under_exec (GtkWidget * w, GtkFileSelection * fs)
{
  FILE *infile;
  char bufferp[255];
  int i;
  gchar *path;

  path = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

  if (path[strlen (path) - 1] == '/')
    {				/* checks if it a directory path instead of a file path */
      merror ("No file has been selected !");
      return;
    }

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened");
      return;
    }

  sprintf (projectname, "%s", path);

  if ((infile = fopen (projectname, "wb+")) == NULL)
    {
      merror ("Can't save project !");
      return;
    }

  fprintf (infile, "%s\n", "#WebDesigner_project_file");

  for (i = 1; i < (numprjfiles + 1); i++)
    {
      fprintf (infile, "%s\n", projectfiles[i].path);
    }

  fclose (infile);
  prjsaved = TRUE;
  prjnew = FALSE;
  sprintf (bufferp, "Project : %s", projectname);
  gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, bufferp);

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
}				/* fin de save_project_under_exec() */


void
save_project (GtkWidget * w, int opcode)
{
  FILE *infile;
  int i;

  if (prjopnd == FALSE)
    {
      merror (_("No file is currently opened"));
      return;
    }

  if (prjsaved == TRUE) return;

  if (prjnew == TRUE) opcode = 1;

  switch (opcode)
    {
    case 0:
      if ((infile = fopen (projectname, "wb")) == NULL)
	{
	  merror ("Can't save project !");
	  return;
	}

      fprintf (infile, "%s\n", "#WebDesigner_project_file");

      for (i = 1; i < (numprjfiles + 1); i++)
	{
	  fprintf (infile, "%s\n", projectfiles[i].path);
	}
      fclose (infile);
      prjsaved = TRUE;
      return;
      break;

    case 1:
      filew = gtk_file_selection_new ("Save project as...");

      /* Connect the ok_button to file_ok_sel function */
      gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		 "clicked", (GtkSignalFunc) save_project_under_exec, filew);

      /* Connect the cancel_button to destroy the widget */
      gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
      gtk_signal_connect_object_after (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

      /* Let's set the default filename */
      gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "project1.prj");

      gtk_widget_show (filew);
      return;
      break;
    }
}				/* fin de save_project(void) */



void
save_project_under_dialog (void)
{

  if (prjopnd == FALSE)
    {
      merror ("Aucun projet n'est actuellement ouvert ");
      return;
    }

  filew = gtk_file_selection_new ("Save project as...");

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		 "clicked", (GtkSignalFunc) save_project_under_exec, filew);

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_signal_connect_object_after (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));

  /* Lets set the default filename */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filew), "project1.prj");

  gtk_widget_show (filew);

}				/* fin de save_project_under_dialog() */


void
add_file (void)
{

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened ");
      return;
    }

  if (sheet[curfile].acces == FALSE)
    {
      merror ("The file is read only !");
      return;
    }

  if (sheet[curfile].new == TRUE)
    {
      merror ("Can't add an unsaved file !");
      return;
    }

  if (sheet[curfile].inprj == TRUE)
    {
      merror ("File is already in project !");
      return;
    }
  else
    sheet[curfile].inprj = TRUE;

  if (prjsaved == TRUE)
    prjsaved = FALSE;
  numprjfiles = numprjfiles + 1;
  sprintf (projectfiles[numprjfiles].path, "%s", sheet[curfile].chemin);

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }

}				/* fin de add_file(); */



int
updateprjlist (void)
{
  int i, sve = 0;

  for (i = 1; i < (numprjfiles + 1); i++)
    {

      if (!strcmp (projectfiles[i].path, sheet[curfile].chemin))
	{
	  sve = i;
	}
    }

  if (sve == numprjfiles)
    return 0;
  else
    {
      for (i = sve; i < (numprjfiles + 1); i++)
	{
	  sprintf (projectfiles[i].path, "%s", projectfiles[i + 1].path);
	}
    }

  return 1;
}				/* fin de updateprjlist() */




void
remove_file (void)
{

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened ");
      return;
    }

  if (sheet[curfile].inprj == FALSE)
    {
      merror ("File is not in project ! ");
      return;
    }
  else
    {
      sheet[curfile].inprj = FALSE;
    }

  if (prjsaved == TRUE)
    prjsaved = FALSE;

  updateprjlist ();
  numprjfiles = numprjfiles - 1;


  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }

}				/* fin de remove_file(); */


void
remove_file2 (void)
{
  int i, sve;

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened");
      return;
    }

  if (prjsaved == TRUE)
    prjsaved = FALSE;

  sve = clist_row2;

  if (sve == numprjfiles + 1)
    goto suite;
  else
    {
      for (i = sve; i < (numprjfiles + 1); i++)
	{
	  sprintf (projectfiles[i].path, "%s", projectfiles[i + 1].path);
	}
    }

suite:
  numprjfiles = numprjfiles - 1;

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
}				/* fin de remove_file2(); */



void
open_all_prjfiles (void)
{
  int i;

  for (i = 1; i < numprjfiles + 1; i++)
    {
      openfile2 (projectfiles[i].path, FALSE, TRUE);
    }

}				/* fin de open_all_prjfiles */



void
open_project (GtkWidget * w, GtkFileSelection * fs)
{
  FILE *infile2;
  gchar *buf;
  gchar *inter;
  char bufferp[255];

  inter = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

    if (inter[strlen (inter) - 1] == '/')
    {				/*checks if it is a directory path instead of a file path */
      gtk_widget_destroy (filew);
      merror ("No file has been selected !");
      return;
    }

  if ((access (inter, R_OK)) != 0)
    {
      merror ("Can't open project !");
      return;
    }

  sprintf (projectname, "%s", inter);

  infile2 = fopen (projectname, "r");

  buf = (char *) malloc (1024);

  fscanf (infile2, "%s", buf);

/* vrification qu'il s'agit d'un fichier de projet valide */

  if (strcmp (buf, "#WebDesigner_project_file"))
    {
      merror ("Invalid or corrupted project file");
      free (buf);
      fclose (infile2);
      return;
    }

  while (fscanf (infile2, "%s", buf) > 0)
    {

      if ((access (buf, R_OK)) != 0)
	{
	  sprintf (bufferp, "Can't open file %s !", buf);
	  merror (bufferp);
	  goto suit;
	}

      numprjfiles = numprjfiles + 1;
      sprintf (projectfiles[numprjfiles].path, "%s", buf);
    suit:
    }

  fclose (infile2);
  free (buf);

  prjsaved = TRUE;
  prjopnd = TRUE;
  prjnew = FALSE;

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
  else
    {
      init_clist ();
    }
}				/* fin de open_project() */



		/* pour la ligne de commande */
int
open_project2 (gchar * inter)
{
  FILE *infile2;
  gchar *buf;
  char bufferp[255];

  sprintf (projectname, "%s", inter);

  buf = (char *) malloc (512);

  if ((infile2 = fopen (projectname, "rb")) == NULL)
    {
      merror ("Can't open project !");
      free (buf);
      sprintf (projectname, "%s", "aucun");
      return -1;
    }

  fscanf (infile2, "%s", buf);

/* vrification qu'il s'agit d'un fichier de projet valide */
  if (strcmp (buf, "#WebDesigner_project_file"))
    {
      merror ("Invalid or corrupted project file");
      free (buf);
      fclose (infile2);
      sprintf (projectname, "%s", "aucun");
      return -1;
    }


  while (fscanf (infile2, "%s", buf) > 0)
    {

      if ((access (buf, R_OK)) != 0)
	{
	  sprintf (bufferp, "Can't open file %s !", buf);
	  merror (bufferp);
	  goto suit2;
	}

      numprjfiles = numprjfiles + 1;
      sprintf (projectfiles[numprjfiles].path, "%s", buf);
    suit2:
    }

  fclose (infile2);
  free (buf);

  prjsaved = TRUE;
  prjopnd = TRUE;
  prjnew = FALSE;

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
  else
    {
      init_clist ();
    }
  return 1;
}				/* fin de open_project2() */




void
open_project_dialog (void)
{
  /* Create a new file selection widget */
  filew = gtk_file_selection_new ("Open project...");

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", (GtkSignalFunc) open_project, filew);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show (filew);


  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button), "clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (filew));
  gtk_widget_show (filew);

}				/* fin de open_project_dialog() */


gboolean
searchin ()
{
  int i;

  for (i = 0; i < numfileopened; i++)
    {
      gtk_notebook_set_page (GTK_NOTEBOOK (notebook), i);
      if (sheet[i].inprj == TRUE)
	{
	  close_file ();
	  return TRUE;
	}
    }
  return FALSE;
}				/* fin de searchin */


void
close_all_prjfiles (void)
{
  gboolean inproject = TRUE;

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened ");
      return;
    }

  while (inproject)
    {
      inproject = searchin ();
    }

}				/* fin de close_all_prjfiles() */



void
close_project (void)
{
  char bufferp[255];

  if (prjopnd == FALSE)
    {
      merror ("No project is currently opened ");
      return;
    }

  sprintf (projectname, "%s", "none");
  prjsaved = TRUE;
  prjopnd = FALSE;
  numprjfiles = 0;

  sprintf (bufferp, "Current project : %s", projectname);
  gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, bufferp);

  if (GTK_IS_CLIST (clist))
    {
      refresh_clist ();
    }
}				/* fin de close_project() */
